@@ -131,6 +131,9 @@ init_globals (void)
           g_free (custom_dirs);
         }
 
+      if (override_search_path != NULL)
+        override_search_path = g_slist_reverse (override_search_path);
+
       libdir = GOBJECT_INTROSPECTION_LIBDIR;
 
       typelib_dir = g_build_filename (libdir, "girepository-1.0", NULL);
@@ -926,9 +929,9 @@ g_irepository_get_typelib_path (GIRepository *repository,
 static GMappedFile *
 find_namespace_version (const gchar  *namespace,
 			const gchar  *version,
+			GSList       *search_path,
 			gchar       **path_ret)
 {
-  GSList *tmp_path;
   GSList *ldir;
   GError *error = NULL;
   GMappedFile *mfile = NULL;
@@ -936,8 +939,7 @@ find_namespace_version (const gchar  *namespace,
 
   fname = g_strdup_printf ("%s-%s.typelib", namespace, version);
 
-  tmp_path = build_search_path_with_overrides ();
-  for (ldir = tmp_path; ldir; ldir = ldir->next)
+  for (ldir = search_path; ldir; ldir = ldir->next)
     {
       char *path = g_build_filename (ldir->data, fname, NULL);
 
@@ -952,7 +954,6 @@ find_namespace_version (const gchar  *namespace,
       break;
     }
   g_free (fname);
-  g_slist_free (tmp_path);
   return mfile;
 }
 
@@ -1046,29 +1047,23 @@ free_candidate (struct NamespaceVersionCandidadate *candidate)
   g_slice_free (struct NamespaceVersionCandidadate, candidate);
 }
 
-static GMappedFile *
-find_namespace_latest (const gchar  *namespace,
-		       gchar       **version_ret,
-		       gchar       **path_ret)
+static GSList *
+enumerate_namespace_versions (const gchar *namespace,
+			      GSList      *search_path)
 {
-  GSList *tmp_path;
-  GSList *ldir;
-  GError *error = NULL;
+  GSList *candidates = NULL;
+  GHashTable *found_versions = g_hash_table_new (g_str_hash, g_str_equal);
   char *namespace_dash;
   char *namespace_typelib;
-  GSList *candidates = NULL;
-  GMappedFile *result = NULL;
+  GSList *ldir;
+  GError *error = NULL;
   int index;
 
-  *version_ret = NULL;
-  *path_ret = NULL;
-
   namespace_dash = g_strdup_printf ("%s-", namespace);
   namespace_typelib = g_strdup_printf ("%s.typelib", namespace);
 
   index = 0;
-  tmp_path = build_search_path_with_overrides ();
-  for (ldir = tmp_path; ldir; ldir = ldir->next)
+  for (ldir = search_path; ldir; ldir = ldir->next)
     {
       GDir *dir;
       const char *dirname;
@@ -1102,6 +1097,10 @@ find_namespace_latest (const gchar  *namespace,
 	  else
 	    continue;
 
+	  if (g_hash_table_lookup (found_versions, version) != NULL)
+	    continue;
+	  g_hash_table_insert (found_versions, version, version);
+
 	  path = g_build_filename (dirname, entry, NULL);
 	  mfile = g_mapped_file_new (path, FALSE, &error);
 	  if (mfile == NULL)
@@ -1122,6 +1121,27 @@ find_namespace_latest (const gchar  *namespace,
       index++;
     }
 
+  g_free (namespace_dash);
+  g_free (namespace_typelib);
+  g_hash_table_destroy (found_versions);
+
+  return candidates;
+}
+
+static GMappedFile *
+find_namespace_latest (const gchar  *namespace,
+		       GSList       *search_path,
+		       gchar       **version_ret,
+		       gchar       **path_ret)
+{
+  GSList *candidates;
+  GMappedFile *result = NULL;
+
+  *version_ret = NULL;
+  *path_ret = NULL;
+
+  candidates = enumerate_namespace_versions (namespace, search_path);
+
   if (candidates != NULL)
     {
       struct NamespaceVersionCandidadate *elected;
@@ -1138,34 +1158,47 @@ find_namespace_latest (const gchar  *namespace,
       g_slist_foreach (candidates, (GFunc) free_candidate, NULL);
       g_slist_free (candidates);
     }
-  g_free (namespace_dash);
-  g_free (namespace_typelib);
-  g_slist_free (tmp_path);
   return result;
 }
 
 /**
- * g_irepository_require:
- * @repository: (allow-none): Repository, may be %NULL for the default
- * @namespace_: GI namespace to use, e.g. "Gtk"
- * @version: (allow-none): Version of namespace, may be %NULL for latest
- * @flags: Set of %GIRepositoryLoadFlags, may be 0
- * @error: a #GError.
+ * g_irepository_enumerate_versions:
+ * @repository: (allow-none): the repository
+ * @namespace_: GI namespace, e.g. "Gtk"
  *
- * Force the namespace @namespace_ to be loaded if it isn't already.
- * If @namespace_ is not loaded, this function will search for a
- * ".typelib" file using the repository search path.  In addition, a
- * version @version of namespace may be specified.  If @version is
- * not specified, the latest will be used.
+ * Obtain a list of versions for @namespace_ in this @repository.
  *
- * Returns: a pointer to the #GTypelib if successful, %NULL otherwise
+ * Returns: (element-type utf8) (transfer full): the array of versions.
  */
-GTypelib *
-g_irepository_require (GIRepository  *repository,
-		       const gchar   *namespace,
-		       const gchar   *version,
-		       GIRepositoryLoadFlags flags,
-		       GError       **error)
+GList *
+g_irepository_enumerate_versions (GIRepository *repository,
+			 const gchar  *namespace_)
+{
+  GList *ret = NULL;
+  GSList *search_path;
+  GSList *candidates, *link;
+
+  search_path = build_search_path_with_overrides ();
+  candidates = enumerate_namespace_versions (namespace_, search_path);
+  g_slist_free (search_path);
+
+  for (link = candidates; link; link = link->next)
+    {
+      struct NamespaceVersionCandidadate *candidate = link->data;
+      ret = g_list_append (ret, g_strdup (candidate->version));
+      free_candidate (candidate);
+    }
+  g_slist_free (candidates);
+  return ret;
+}
+
+static GTypelib *
+require_internal (GIRepository  *repository,
+		  const gchar   *namespace,
+		  const gchar   *version,
+		  GIRepositoryLoadFlags flags,
+		  GSList        *search_path,
+		  GError       **error)
 {
   GMappedFile *mfile;
   GTypelib *ret = NULL;
@@ -1198,12 +1231,14 @@ g_irepository_require (GIRepository  *repository,
 
   if (version != NULL)
     {
-      mfile = find_namespace_version (namespace, version, &path);
+      mfile = find_namespace_version (namespace, version,
+				      search_path, &path);
       tmp_version = g_strdup (version);
     }
   else
     {
-      mfile = find_namespace_latest (namespace, &tmp_version, &path);
+      mfile = find_namespace_latest (namespace, search_path,
+				     &tmp_version, &path);
     }
 
   if (mfile == NULL)
@@ -1221,7 +1256,19 @@ g_irepository_require (GIRepository  *repository,
       goto out;
     }
 
-  typelib = g_typelib_new_from_mapped_file (mfile);
+  {
+    GError *temp_error = NULL;
+    typelib = g_typelib_new_from_mapped_file (mfile, &temp_error);
+    if (!typelib)
+      {
+	g_set_error (error, G_IREPOSITORY_ERROR,
+		     G_IREPOSITORY_ERROR_TYPELIB_NOT_FOUND,
+		     "Failed to load typelib file '%s' for namespace '%s': %s",
+		     path, namespace, temp_error->message);
+	g_clear_error (&temp_error);
+	goto out;
+      }
+  }
   header = (Header *) typelib->data;
   typelib_namespace = g_typelib_get_string (typelib, header->namespace);
   typelib_version = g_typelib_get_string (typelib, header->nsversion);
@@ -1233,6 +1280,7 @@ g_irepository_require (GIRepository  *repository,
 		   "Typelib file %s for namespace '%s' contains "
 		   "namespace '%s' which doesn't match the file name",
 		   path, namespace, typelib_namespace);
+      g_typelib_free (typelib);
       goto out;
     }
   if (version != NULL && strcmp (typelib_version, version) != 0)
@@ -1242,6 +1290,7 @@ g_irepository_require (GIRepository  *repository,
 		   "Typelib file %s for namespace '%s' contains "
 		   "version '%s' which doesn't match the expected version '%s'",
 		   path, namespace, typelib_version, version);
+      g_typelib_free (typelib);
       goto out;
     }
 
@@ -1258,6 +1307,71 @@ g_irepository_require (GIRepository  *repository,
   return ret;
 }
 
+/**
+ * g_irepository_require:
+ * @repository: (allow-none): Repository, may be %NULL for the default
+ * @namespace_: GI namespace to use, e.g. "Gtk"
+ * @version: (allow-none): Version of namespace, may be %NULL for latest
+ * @flags: Set of %GIRepositoryLoadFlags, may be 0
+ * @error: a #GError.
+ *
+ * Force the namespace @namespace_ to be loaded if it isn't already.
+ * If @namespace_ is not loaded, this function will search for a
+ * ".typelib" file using the repository search path.  In addition, a
+ * version @version of namespace may be specified.  If @version is
+ * not specified, the latest will be used.
+ *
+ * Returns: a pointer to the #GTypelib if successful, %NULL otherwise
+ */
+GTypelib *
+g_irepository_require (GIRepository  *repository,
+		       const gchar   *namespace,
+		       const gchar   *version,
+		       GIRepositoryLoadFlags flags,
+		       GError       **error)
+{
+  GSList *search_path;
+  GTypelib *typelib;
+
+  search_path = build_search_path_with_overrides ();
+  typelib = require_internal (repository, namespace, version, flags,
+			      search_path, error);
+  g_slist_free (search_path);
+
+  return typelib;
+}
+
+/**
+ * g_irepository_require_private:
+ * @repository: (allow-none): Repository, may be %NULL for the default
+ * @typelib_dir: Private directory where to find the requested typelib
+ * @namespace_: GI namespace to use, e.g. "Gtk"
+ * @version: (allow-none): Version of namespace, may be %NULL for latest
+ * @flags: Set of %GIRepositoryLoadFlags, may be 0
+ * @error: a #GError.
+ *
+ * Force the namespace @namespace_ to be loaded if it isn't already.
+ * If @namespace_ is not loaded, this function will search for a
+ * ".typelib" file within the private directory only. In addition, a
+ * version @version of namespace should be specified.  If @version is
+ * not specified, the latest will be used.
+ *
+ * Returns: a pointer to the #GTypelib if successful, %NULL otherwise
+ */
+GTypelib *
+g_irepository_require_private (GIRepository  *repository,
+			       const gchar   *typelib_dir,
+			       const gchar   *namespace,
+			       const gchar   *version,
+			       GIRepositoryLoadFlags flags,
+			       GError       **error)
+{
+  GSList search_path = { (gpointer) typelib_dir, NULL };
+
+  return require_internal (repository, namespace, version, flags,
+			   &search_path, error);
+}
+
 static gboolean
 g_irepository_introspect_cb (const char *option_name,
 			     const char *value,
@@ -1294,6 +1408,14 @@ g_irepository_error_quark (void)
   return quark;
 }
 
+/**
+ * g_type_tag_to_string:
+ * @type: the type_tag
+ *
+ * Obtain a string representation of @type
+ *
+ * Returns: the string
+ */
 const gchar*
 g_type_tag_to_string (GITypeTag type)
 {
@@ -1345,3 +1467,61 @@ g_type_tag_to_string (GITypeTag type)
       return "unknown";
     }
 }
+
+/**
+ * g_info_type_to_string:
+ * @type: the info type
+ *
+ * Obtain a string representation of @type
+ *
+ * Returns: the string
+ */
+const gchar*
+g_info_type_to_string (GIInfoType type)
+{
+  switch (type)
+    {
+    case GI_INFO_TYPE_INVALID:
+      return "invalid";
+    case GI_INFO_TYPE_FUNCTION:
+      return "function";
+    case GI_INFO_TYPE_CALLBACK:
+      return "callback";
+    case GI_INFO_TYPE_STRUCT:
+      return "struct";
+    case GI_INFO_TYPE_BOXED:
+      return "boxed";
+    case GI_INFO_TYPE_ENUM:
+      return "enum";
+    case GI_INFO_TYPE_FLAGS:
+      return "flags";
+    case GI_INFO_TYPE_OBJECT:
+      return "object";
+    case GI_INFO_TYPE_INTERFACE:
+      return "interface";
+    case GI_INFO_TYPE_CONSTANT:
+      return "constant";
+    case GI_INFO_TYPE_ERROR_DOMAIN:
+      return "error domain";
+    case GI_INFO_TYPE_UNION:
+      return "union";
+    case GI_INFO_TYPE_VALUE:
+      return "value";
+    case GI_INFO_TYPE_SIGNAL:
+      return "signal";
+    case GI_INFO_TYPE_VFUNC:
+      return "vfunc";
+    case GI_INFO_TYPE_PROPERTY:
+      return "property";
+    case GI_INFO_TYPE_FIELD:
+      return "field";
+    case GI_INFO_TYPE_ARG:
+      return "arg";
+    case GI_INFO_TYPE_TYPE:
+      return "type";
+    case GI_INFO_TYPE_UNRESOLVED:
+      return "unresolved";
+    default:
+      return "unknown";
+  }
+}
