@@ -22,7 +22,7 @@ from __future__ import with_statement
 
 from contextlib import contextmanager
 from cStringIO import StringIO
-from xml.sax.saxutils import quoteattr
+from xml.sax.saxutils import escape, quoteattr
 
 from .libtoolimporter import LibtoolImporter
 
@@ -67,7 +67,7 @@ def collect_attributes(tag_name, attributes, self_indent,
     return attr_value
 
 
-with LibtoolImporter:
+with LibtoolImporter(None, None):
     from giscanner._giscanner import collect_attributes
 
 
@@ -84,6 +84,8 @@ class XMLWriter(object):
     # Private
 
     def _open_tag(self, tag_name, attributes=None):
+        if attributes is None:
+            attributes = []
         attrs = collect_attributes(
             tag_name, attributes, self._indent,
             self._indent_char,
@@ -109,7 +111,7 @@ class XMLWriter(object):
             attributes = []
         prefix = '<%s' % (tag_name, )
         if data is not None:
-            suffix = '>%s</%s>' % (data, tag_name)
+            suffix = '>%s</%s>' % (escape(data), tag_name)
         else:
             suffix = '/>'
         attrs = collect_attributes(
@@ -120,6 +122,8 @@ class XMLWriter(object):
         self.write_line(prefix + attrs + suffix)
 
     def push_tag(self, tag_name, attributes=None):
+        if attributes is None:
+            attributes = []
         self._open_tag(tag_name, attributes)
         self._tag_stack.append(tag_name)
         self._indent += self._indent_unit
